package jalview.gui;

import jalview.jbgui.GUserDefinedColours;
import jalview.datamodel.*;
import jalview.io.*;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.event.*;
import jalview.schemes.*;
import java.io.*;
import java.util.*;


public class UserDefinedColours extends GUserDefinedColours implements ChangeListener
{

  AlignmentPanel ap;
  SequenceGroup seqGroup;
  JButton selectedButton;
  Vector oldColours = new Vector();
  ColourSchemeI oldColourScheme;
  JInternalFrame frame;


  public UserDefinedColours(JInternalFrame f, AlignmentPanel ap, SequenceGroup sg)
  {
    super();
    colorChooser.getSelectionModel().addChangeListener(this);
    this.ap = ap;
    frame = f;
    seqGroup = sg;


      if (seqGroup != null)
        oldColourScheme = seqGroup.cs;
      else
        oldColourScheme = ap.av.getGlobalColourScheme();

      for (int i = 0; i < 20; i++)
        makeButton(ResidueProperties.aa2Triplet.get(ResidueProperties.aa[i]) +
                   "", ResidueProperties.aa[i]);

      makeButton("B", "B");
      makeButton("Z", "Z");
      makeButton("X", "X");
      makeButton("'.','-',' '", "-");

      if(jalview.bin.Cache.getProperty("USER_DEFINED_COLOUR")!=null)
      {
        loadColour(jalview.bin.Cache.getProperty("USER_DEFINED_COLOUR"));
      }


  }

 public void stateChanged(ChangeEvent evt)
 {
   if(selectedButton!=null)
     selectedButton.setBackground( colorChooser.getColor() );
 }

  public void colourButtonPressed(MouseEvent e)
  {
     selectedButton = (JButton)e.getSource();
     colorChooser.setColor(selectedButton.getBackground());
  }

  void makeButton(String label, String aa)
  {
    final JButton button = new JButton();
    Color col = Color.white;


    try{
      col = oldColourScheme.findColour(aa, -1);
    }catch(Exception ex){}

    button.setBackground(  col );
    oldColours.addElement( col );
    button.setText( label );
    button.setForeground( col.darker().darker().darker() );
    button.setFont(new java.awt.Font("Verdana", 1, 10));
    button.addMouseListener(new java.awt.event.MouseAdapter()
    {
      public void mouseClicked(MouseEvent e)
      {
        colourButtonPressed(e);
      }
    });

    buttonPanel.add(button);
  }


  protected void okButton_actionPerformed(ActionEvent e)
  {
    applyButton_actionPerformed(null);
    try{
      frame.setClosed(true);
    }catch(Exception ex){}
  }

  protected void applyButton_actionPerformed(ActionEvent e)
  {
    UserColourScheme ucs = new UserColourScheme();
    Color [] newColours = new Color[24];
    for(int i=0; i<24; i++)
    {
          JButton button = (JButton)buttonPanel.getComponent(i);
          newColours[i] = button.getBackground();
    }

    ucs.setThreshold(0);
    ucs.setColourScheme( newColours );

    if(seqGroup!=null)
      seqGroup.cs = ucs;
    else
      ap.av.setGlobalColourScheme(ucs);

    ap.RefreshPanels();
  }

  protected void loadbutton_actionPerformed(ActionEvent e)
  {
    JalviewFileChooser chooser = new JalviewFileChooser(jalview.bin.Cache.getProperty("LAST_DIRECTORY"),
        new String[]{".jvuc"}, "Jalview User Colours");
    chooser.setFileView(new jalview.io.JalviewFileView());
    chooser.setDialogTitle("Load colour scheme");
    chooser.setToolTipText("Load");
    int value = chooser.showOpenDialog(this);
    if (value == JalviewFileChooser.APPROVE_OPTION)
    {
      String choice = chooser.getSelectedFile().getPath();
      jalview.bin.Cache.setProperty("LAST_DIRECTORY",  choice);
      loadColour(choice);
    }

  }

  void loadColour(String file)
  {
    try
    {
      BufferedReader in = new BufferedReader(new FileReader(file));
      for (int i = 0; i < 24; i++)
      {
        JButton button = (JButton) buttonPanel.getComponent(i);
        Color c = new Color(Integer.parseInt(in.readLine()));
        button.setBackground(c);
      }
    }
    catch (Exception ex)
    {}

  }

  protected void savebutton_actionPerformed(ActionEvent e)
  {
    JalviewFileChooser chooser = new JalviewFileChooser(jalview.bin.Cache.getProperty("LAST_DIRECTORY"));
    chooser.setFileView(new jalview.io.JalviewFileView());
    chooser.setDialogTitle("Save colour scheme");
    chooser.setToolTipText("Save");
    int value = chooser.showSaveDialog(this);
    if (value == JalviewFileChooser.APPROVE_OPTION)
    {
      String choice = chooser.getSelectedFile().getPath();
      jalview.bin.Cache.setProperty("USER_DEFINED_COLOUR", choice);

      try{
        PrintWriter out = new PrintWriter(new FileWriter(choice));
        for(int i=0; i<24; i++)
        {
          JButton button = (JButton)buttonPanel.getComponent(i);
          out.println(button.getBackground().getRGB() + "");
        }

        out.close();
      }
      catch(Exception ex)
      {ex.printStackTrace();}
    }

  }

  protected void cancelButton_actionPerformed(ActionEvent e)
  {
    UserColourScheme ucs = new UserColourScheme();
    Color [] newColours = new Color[24];
    for(int i=0; i<24; i++)
    {
          newColours[i] = (Color)oldColours.elementAt(i);
          buttonPanel.getComponent(i).setBackground(newColours[i]);
    }

    ucs.setColourScheme( newColours );
    if (seqGroup != null)
      seqGroup.cs = ucs;
    else
      ap.av.setGlobalColourScheme(ucs);

    ap.RefreshPanels();

  }


}
